/*
libWTF v1.0

by Chih-Yun Tsai (chi.tsai@rwth-aachen.de)
Computer Vision Group
RWTH Aachen University
 
Implementation of the approach presented in:
 
Fixing WTFs: Detecting Image Matches caused by Watermarks, Timestamps,
and Frames in Internet Photos
Tobias Weyand, Chih-Yun Tsai, Bastian Leibe
in WACV 2015.

Licensed under GPLv3.

See README.TXT for compilation and usage instructions.
*/

#ifndef LIBWTF_H
#define LIBWTF_H

#include "def.h"

#include <vector>
#include <opencv2/opencv.hpp>

namespace wtf {

/*
This function builds and returns the 2-way feature vectors, according
to a given image pair, their inliers and parameters.
 
Use this function if your interest region detector outputs circles,
i.e. one (x,y,s) tuple per interest region. Use the below version if
you have ellipses.

For an image pair <img1,img2>, the following data is required:
- [homography] specifies the 3-by-3 homography matrix projecting from img2 onto img1.
               The matrix must be stored in this vector row-wise, meaning the first 
               three elements of [homography] specify the first row of the homography 
               matrix.
- [w1] and [h1] specify the size of img1, namely width and height respectively.
- [x1s] and [y1s] specify the inliers in img1's coordinate.
- [s1s] specifies the inliers' radius in img1's coordinate.
- [w2], [h2], [x2], [y2] and [s2s] are associated with img2.
This function then computes the spatial histogram of the inlier regions and its
coverage, to build up the feature vector. The histogram has two parameters:
- [histogram_shape] specifies the spatial histogram shape used to compute the feature 
                    vector (see paper). Possible values are:
  enum HistogramShape{
    DIST2CENTER,
    DIST2BORDER
  };
- [n_histogram_bins] specifies how many bins the constructed histogram has.

This function returns a pair of vector<float>, where the first one is the feature
vector constructed from img1 and its inliers, and the second one - constructed from
img2 and its inliers.
*/
bool extract_feature_vector(
    const std::vector<double> &homography,	// row-wise
    const double w1,
    const double h1,
    const double w2,
    const double h2,
    const HistogramShape histogram_shape,
    const int n_histogram_bins,
    const std::vector<double> &x1s,
    const std::vector<double> &y1s,
    const std::vector<double> &s1s,
    const std::vector<double> &x2s,
    const std::vector<double> &y2s,
    const std::vector<double> &s2s,
    std::pair<std::vector<float>,std::vector<float> > &feature_vector
);

/*
This function builds and returns the 2-way feature vectors, according
to a given image pair, their inliers and parameters.
 
This is a version of the above function that takes elliptical feature
regions as input. Here, each local feature is specified by a (x,y,a,b,c)
tuple. See above for a description of its parameters.
*/
bool extract_feature_vector(
    const std::vector<double> &homography,	// row-wise
    const double w1,
    const double h1,
    const double w2,
    const double h2,
    const HistogramShape histogram_shape,
    const int n_histogram_bins,
    const std::vector<double> &x1s,
    const std::vector<double> &y1s,
    const std::vector<double> &a1s,
    const std::vector<double> &b1s,
    const std::vector<double> &c1s,
    const std::vector<double> &x2s,
    const std::vector<double> &y2s,
    const std::vector<double> &a2s,
    const std::vector<double> &b2s,
    const std::vector<double> &c2s,
    std::pair<std::vector<float>,std::vector<float> > &feature_vector
);

/*
This function classifies a feature vector pair (Two-way Matching), and
returns the classification score, given a trained model.
The lower score from the feature vector pair is returned (Two-way Matching.)
A negative score means non-WTF, a positive score means WTF.
*/
float classify (
    const std::pair<std::vector<float>,std::vector<float> > &v,
    const CvBoost * const model );

/*
This function trains and returns the libWTF model, on the given feature
vectors. [n_itr] specifies the number of weak classifiers used in the
(AdaBoost) model.
*/
CvBoost* train (
    std::vector<std::vector<float> > v,
    std::vector<bool> l,
    const int n_itr=DEFAULT_ITR );

}  // namespace wtf

#endif
