#include <iostream>
#include <string>
#include <fstream>
#include <vector>
#include <opencv2/opencv.hpp>
#include <stdio.h>

#include <libwtf.h>

using namespace std;
using namespace cv;
using namespace wtf;

HistogramShape histogramShape;
int n_bins;

string modelPath;
string inliersPath;

pair<vector<float>,vector<float> > features;

struct Inliers {
    vector<double> homography;
    double w1;
    double h1;
    double w2;
    double h2;
    vector<double> x1s;
    vector<double> y1s;
    vector<double> a1s;
    vector<double> b1s;
    vector<double> c1s;
    vector<double> x2s;
    vector<double> y2s;
    vector<double> a2s;
    vector<double> b2s;
    vector<double> c2s;
} ;

bool read_inliers(Inliers& inliers, const string& inliers_file_path) ;
string readline(ifstream * inFile) ;

bool exist(const string& name)
{
    ifstream file(name);
    return bool(file);
}

int main(int argc, char** argv)
{
    if (argc!=5) {
        printf(
                "usage:    $ demoLibWTF <Inliers_Path> <Model_Path> <Histogram_Shape> <Number_of_Bins>\n"
                "where\n"
                "    <Inliers_Path> specifies the path of inliers file, which locates in /libwtf/res/inliers.*.txt for example,\n"
                "    <Model_Path> specifies the prebuilt model file,\n"
                "    <Histogram_Shape> [DIST2BORDER, DIST2CENTER] specifies how the inlier region is considered to build the features. It must match the features used to train the model.\n"
                "    <Number_of_Bins> specifies how the inlier region is considered to build the features. It must match the features used to train the model.\n"
        );
        return -1;
    }
    inliersPath = argv[1];
    modelPath = argv[2];
    string shapeStr = argv[3];
    double score_threshold = -1.8; // Tuned for 99% true-positive rate
    if (!shapeStr.compare("DIST2BORDER")) {
        histogramShape = DIST2BORDER;
    } else if (!shapeStr.compare("DIST2CENTER")) {
        histogramShape = DIST2CENTER;
    } else {
        cerr << "Invalid histogram shape: " << shapeStr << endl;
        return -1;
    }
    try {
        n_bins = stod(argv[4]);
    } catch (invalid_argument& ia) {
        cerr << "Invalid number of bins: " << argv[4] <<endl;
        return -1;
    }

    if (!exist(modelPath)) {
        cerr << "model doesn't exist: " <<modelPath <<endl;
        return -1;
    }
    if (!exist(inliersPath)) {
        cerr << "inliers file doesn't exist: " <<inliersPath <<endl;
        return -1;
    }

    Inliers inliers;
    if (!read_inliers(inliers,inliersPath)) {
        cerr << "Failed to read inliers file: " <<inliersPath <<endl;
        return -1;
    }

    cout << "Extracting feature vectors from "<<inliersPath<<"..."<<endl;
    extract_feature_vector (
            inliers.homography, inliers.w1, inliers.h1, inliers.w2, inliers.h2,
            histogramShape, n_bins,
            inliers.x1s, inliers.y1s, inliers.a1s, inliers.b1s, inliers.c1s,
            inliers.x2s, inliers.y2s, inliers.a2s, inliers.b2s, inliers.c2s,
            features );
    //    cout << Mat(features.first) <<endl;
    //    cout << Mat(features.second) <<endl;

    cout << "Classifying the feature vector by model: "<<modelPath<<"..."<<endl;
    CvBoost * model = new CvBoost();
    model->load(modelPath.c_str());
    double score = classify( features, model );
    cout << "classification score = "<< score << " (score threshold = " << score_threshold << ")" << endl;
    if(score > score_threshold) {
      cout << "Classified as WTF" << endl;
    } else {
      cout << "Classified as non-WTF" << endl;
    }

    delete model;
    return 0;
}


/* Reads and return the number from a stringStream */
inline double read_num(istringstream& iss) {
    double num;
    iss >> num;
    return num;
}
/* Reads and return the vector<double> from a stringStream */
vector<double> read_vector(istringstream& iss) {
    vector<double> rtn(0);
    double num;
    while(iss>>num) {
        rtn.push_back(num);
    }
    return rtn;
}

bool read_inliers(Inliers& inliers, const string& inliers_file_path) {
    ifstream inliersFile(inliers_file_path);
    string line;
    while (inliersFile.good()) {
        getline(inliersFile, line);

        // Extract the label (first elements of a line.)
        istringstream iss(line);
        string label;
        iss >> label;

        if (!label.length()) {
            // Simply skip a blank line
        } else if (!label.compare("w1")) {
            inliers.w1 = read_num(iss);
        } else if (!label.compare("h1")) {
            inliers.h1 = read_num(iss);
        } else if (!label.compare("w2")) {
            inliers.w2 = read_num(iss);
        } else if (!label.compare("h2")) {
            inliers.h2 = read_num(iss);
        } else if (!label.compare("homography")) {
            inliers.homography = read_vector(iss);
        } else if (!label.compare("x1s")) {
            inliers.x1s = read_vector(iss);
        } else if (!label.compare("y1s")) {
            inliers.y1s = read_vector(iss);
        } else if (!label.compare("a1s")) {
            inliers.a1s = read_vector(iss);
        } else if (!label.compare("b1s")) {
            inliers.b1s = read_vector(iss);
        } else if (!label.compare("c1s")) {
            inliers.c1s = read_vector(iss);
        } else if (!label.compare("x2s")) {
            inliers.x2s = read_vector(iss);
        } else if (!label.compare("y2s")) {
            inliers.y2s = read_vector(iss);
        } else if (!label.compare("a2s")) {
            inliers.a2s = read_vector(iss);
        } else if (!label.compare("b2s")) {
            inliers.b2s = read_vector(iss);
        } else if (!label.compare("c2s")) {
            inliers.c2s = read_vector(iss);
        } else {
            cout << "Invalid data label: " << label <<endl;
            return false;
        }
    }

    return true;
}


